unit FormMain;

interface

uses
	Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Menus,
	ActnList, StdActns, Dialogs;

type
	TForm_Main = class(TForm)
    PopupMenu: TPopupMenu;
    Mnu_Exit: TMenuItem;
    ActionList: TActionList;
    Act_FileExit: TFileExit;
    Act_AlignBottom: TAction;
    Mnu_AlignBottom: TMenuItem;
    Mnu_Sep2: TMenuItem;
    Act_StayOnTop: TAction;
    Mnu_StayOnTop: TMenuItem;
    Mnu_Sep1: TMenuItem;
    Act_ShowScript: TAction;
    Mnu_ShowScript: TMenuItem;
    procedure Act_AlignBottomExecute(Sender: TObject);
		procedure Act_StayOnTopExecute(Sender: TObject);
    procedure FormCreate(Sender: TObject);
		procedure Mnu_DrawItem(Sender: TObject; ACanvas: TCanvas; ARect: TRect; Selected: Boolean);
		procedure Mnu_MeasureItem(Sender: TObject; ACanvas: TCanvas; var Width, Height: Integer);
    procedure FormDestroy(Sender: TObject);
    procedure FormPaint(Sender: TObject);
		procedure FormMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
		procedure FormMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
		procedure FormMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure FormResize(Sender: TObject);
    procedure FormClick(Sender: TObject);
    procedure Act_ShowScriptExecute(Sender: TObject);
    procedure FormCanResize(Sender: TObject; var NewWidth,
      NewHeight: Integer; var Resize: Boolean);
	private
		FAlignBottom: Boolean;
		FSubDroping: Boolean;
		FDroping: Boolean;
		FMousePt: TPoint;
		FBitmap: TBitmap;
	protected
		procedure SetBitmap(Value: TBitmap);
		procedure SetAlignBottom(Value: Boolean);

		procedure BitmapChange(Sender: TObject);
	public
		property Bitmap: TBitmap read FBitmap write SetBitmap;
		property AlignBottom: Boolean read FAlignBottom write SetAlignBottom;
	end;

var
  Form_Main: TForm_Main;
	Running: Boolean = False;

implementation

uses Sub, FormScript, Form_BalloonWindow;

{$R *.dfm}

procedure TForm_Main.FormCreate(Sender: TObject);
begin
	FBitmap := TBitmap.Create;
	FBitmap.OnChange := BitmapChange;
	AlignBottom := True;

	Act_AlignBottom.Checked := AlignBottom;
	Act_StayOnTop.Checked := FormStyle = fsStayOnTop;

	FBitmap.LoadFromFile(ExtractFilePath(Application.ExeName) + 'imo.bmp');
	Left := Screen.WorkAreaRect.Right - Width;
	Running := True;
end;

procedure TForm_Main.FormDestroy(Sender: TObject);
begin
	FBitmap.Free;
end;

procedure TForm_Main.BitmapChange(Sender: TObject);
begin
	Hide;
	try
		SetBounds(Left, Top, FBitmap.Width, FBitmap.Height);
		SetWindowRgn(Handle, CreateBitmapRgn(FBitmap), True);
	finally
		if Running then Show;
	end;
end;

procedure TForm_Main.SetBitmap(Value: TBitmap);
begin
	if Value <> nil then
		FBitmap.Assign(Value)
	else
		ShowMessage('nil C[WŲ');
end;

procedure TForm_Main.SetAlignBottom(Value: Boolean);
begin
	FAlignBottom := Value;
	if FAlignBottom then Resize;
end;

procedure TForm_Main.Act_AlignBottomExecute(Sender: TObject);
begin
	AlignBottom := TAction(Sender).Checked;
end;

procedure TForm_Main.Act_StayOnTopExecute(Sender: TObject);
const
	fsTable: array[False..True] of TFormStyle = (fsNormal, fsStayOnTop);
begin
	FormStyle := fsTable[TAction(Sender).Checked];
	Form_Message.FormStyle := fsTable[TAction(Sender).Checked];
	BitmapChange(Sender);
  Form_Message.ResetWindow;
end;

procedure TForm_Main.Mnu_DrawItem(Sender: TObject; ACanvas: TCanvas; ARect: TRect; Selected: Boolean);

	procedure DrawText(Canvas: TCanvas; Text: String; var ARect: TRect; Flag: Cardinal);
	begin
		Windows.DrawText(Canvas.Handle, PChar(Text), Length(Text), ARect, Flag);
	end;

	procedure DrawText2(Canvas: TCanvas; Text: String; var ARect: TRect; Flag: Cardinal);
	var
		rc: TRect;
	begin
		Canvas.Font.Color := clWhite;
		rc := Rect(ARect.Left - 1, ARect.Top,     ARect.Right - 1, ARect.Bottom);
		DrawText(Canvas, Text, rc, Flag);
		rc := Rect(ARect.Left + 1, ARect.Top,     ARect.Right + 1, ARect.Bottom);
		DrawText(Canvas, Text, rc, Flag);
		rc := Rect(ARect.Left,     ARect.Top - 1, ARect.Right,     ARect.Bottom - 1);
		DrawText(Canvas, Text, rc, Flag);
		rc := Rect(ARect.Left,     ARect.Top + 1, ARect.Right,     ARect.Bottom + 1);
		DrawText(Canvas, Text, rc, Flag);

		Canvas.Font.Color := clBlack;
		DrawText(Canvas, Text, ARect, Flag);
	end;

const
	cmTable: array[False..True] of Integer = (cmSrcCopy, cmNotSrcCopy);
	clTable: array[False..True] of Integer = (clBtnFace, clHighlight);
var
	rc: TRect;
begin
	if not FBitmap.Empty then
	begin
		ACanvas.CopyMode := cmTable[Selected];
		ACanvas.CopyRect(ARect, FBitmap.Canvas, ARect);
	end
	else
	begin
		ACanvas.CopyMode := clTable[Selected];
		ACanvas.FillRect(ARect);
	end;

	if not TMenuItem(Sender).IsLine then
	begin
		ACanvas.Brush.Style := bsClear;

		if TMenuItem(Sender).Checked then
		begin
			rc := ARect;
			Inc(rc.Left, 6);
			DrawText2(ACanvas, 'x', rc, DT_SINGLELINE);
		end;

    rc := ARect;
		Inc(rc.Left, 16);
		Inc(rc.Top,   2);
		DrawText2(ACanvas, TMenuItem(Sender).Caption, rc, DT_SINGLELINE);
	end
	else
	begin
		Inc(ARect.Left);
		Dec(ARect.Right);
		ARect.Top := ARect.Top + 3;
		ARect.Bottom := ARect.Top + 2;
		DrawEdge(ACanvas.Handle, ARect, EDGE_ETCHED, BF_TOP or BF_BOTTOM);
	end;
end;

procedure TForm_Main.Mnu_MeasureItem(Sender: TObject; ACanvas: TCanvas; var Width, Height: Integer);
begin
	if not TMenuItem(Sender).IsLine then
	begin
		Width  := ACanvas.TextWidth(TMenuItem(Sender).Caption) + 12;
		Height := 16;
	end;
end;

procedure TForm_Main.FormPaint(Sender: TObject);
begin
	Canvas.Draw(0, 0, FBitmap);
end;

procedure TForm_Main.FormMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
	if Button = mbLeft then
	begin
		FSubDroping := True;
		FMousePt := Mouse.CursorPos;
	end;
end;

procedure TForm_Main.FormMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
var
	CurPt, pt: TPoint;
begin
	if FSubDroping then
	begin
		pt := ClientToScreen(Point(X, Y));
		if (Abs(FMousePt.X - pt.X) > GetSystemMetrics(SM_CXDRAG)) or
			 (Abs(FMousePt.Y - pt.Y) > GetSystemMetrics(SM_CYDRAG)) then
			FDroping := True;
	end;

	if FDroping then
	begin
		CurPt := Mouse.CursorPos;
		try
			if AlignBottom then
				SetBounds(Left + CurPt.X - FMousePt.X, Screen.WorkAreaRect.Bottom - Height, Width, Height)
			else
				SetBounds(Left + CurPt.X - FMousePt.X, Top + CurPt.Y - FMousePt.Y, Width, Height);
		finally
			FMousePt := CurPt;
		end;
	end;
end;

procedure TForm_Main.FormMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
	FDroping := False;
	FSubDroping := False;
end;

procedure TForm_Main.FormResize(Sender: TObject);
begin
	if AlignBottom then Top := Screen.WorkAreaRect.Bottom - Height;
end;

procedure TForm_Main.FormClick(Sender: TObject);
var
	pt: TPoint;
begin
	if not FDroping then
		if not Form_Message.TimerTalk.Enabled then
		begin
			pt := Mouse.CursorPos;
			pt := ScreenToClient(pt);
			Form_Script.ScriptDispatcher.InvokeMethod('OnClick', [pt.x, pt.y]);
		end;
end;

procedure TForm_Main.Act_ShowScriptExecute(Sender: TObject);
begin
	Form_Script.Visible := TAction(Sender).Checked;
end;

procedure TForm_Main.FormCanResize(Sender: TObject; var NewWidth,
  NewHeight: Integer; var Resize: Boolean);
begin
	if Form_Message <> nil then Form_Message.AlignPos;
end;

end.

