unit Form_BalloonWindow;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls;

type
  TForm_Message = class(TForm)
    TimerHide: TTimer;
    TimerTalk: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure TimerHideTimer(Sender: TObject);
		procedure TimerTalkTimer(Sender: TObject);
    procedure FormPaint(Sender: TObject);
		procedure FormMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
		procedure FormMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure FormClick(Sender: TObject);
	private
		FText, FTextBuf: String;
		FCurPos: TPoint;
	protected
		procedure LineBreak;
		procedure DrawChar(const S: String);
		function ExecuteControlChar(P: PChar): Integer;
	public
		FSkip: Boolean;
		procedure ResetWindow;
		procedure AlignPos;
		procedure SetMessage(const AText: String);
		procedure ClearMessage;            
	end;

const
	DefPos: TPoint = (X: 11; Y: 11);
	LenTable: array[False..True] of Integer = (1, 2); // C in LeadBytes

var
  Form_Message: TForm_Message;

implementation

uses FormMain, Types;

{$R *.dfm}

procedure TForm_Message.ResetWindow;
begin
	SetWindowRgn(Handle, CreateRoundRectRgn(0, 0, Width + 1, Height + 1, 11, 11), True);
end;

procedure TForm_Message.FormCreate(Sender: TObject);
begin
	FSkip := False;
	AlignPos;
  ResetWindow;
	ClearMessage;
end;

procedure TForm_Message.FormDestroy(Sender: TObject);
begin
//
end;

procedure TForm_Message.AlignPos;
begin
	Top  := Form_Main.Top;
	Left := Form_Main.Left - Width - 10;
end;

procedure TForm_Message.TimerHideTimer(Sender: TObject);
begin
	TimerHide.Enabled := False;
	Hide;
end;

procedure TForm_Message.SetMessage(const AText: String);
begin
	TimerHide.Enabled := False;
	FTextBuf := FTextBuf + AText;
	Show;
  TimerTalk.Interval := 30;
	TimerTalk.Enabled := True;
end;

procedure TForm_Message.ClearMessage;
begin
	FCurPos := DefPos;
	TimerTalk.Enabled := False;
	FText := '';
	FTextBuf := '';
	Canvas.Font.Color  := clInfoText;
	Canvas.Font.Height := -12;
	Canvas.Font.Name   := 'lr oSVbN';
	Canvas.Font.Style  := [];
	Canvas.Brush.Color := clInfoBk;
  Canvas.Brush.Style := bsSolid;
	Repaint;
end;

procedure TForm_Message.TimerTalkTimer(Sender: TObject);
var
	s: String;
begin
	if Length(FTextBuf) <> 0 then
	begin
		while (Length(FTextBuf) > 0) and (FTextBuf[1] = '[') do
			Delete(FTextBuf, 1, ExecuteControlChar(PChar(FTextBuf)));

		s := Copy(FTextBuf, 1, LenTable[FTextBuf[1] in LeadBytes]);
		Delete(FTextBuf, 1, LenTable[FTextBuf[1] in LeadBytes]);
		FText := FText + s;
		DrawChar(s);

		TimerTalk.Interval := 30;

		while (Length(FTextBuf) > 0) and (FTextBuf[1] = '[') do
			Delete(FTextBuf, 1, ExecuteControlChar(PChar(FTextBuf)));

		if FSkip then TimerTalk.Interval := 1;
	end;
	if Length(FTextBuf) = 0 then
	begin
		TimerTalk.Enabled := False;
		TimerHide.Enabled := True;
	end;
end;

procedure TForm_Message.LineBreak;
begin
	FCurPos.X := DefPos.X;
	Inc(FCurPos.Y, Abs(Font.Height));
end;

procedure TForm_Message.DrawChar(const S: String);
var
	TextWidth: Integer;
begin
	case S[1] of
		#13: FCurPos.X := DefPos.X;
		#10: Inc(FCurPos.Y, Abs(Font.Height));
		else
		begin
			TextWidth := Canvas.TextWidth(S);
			if (FCurPos.X + TextWidth) > (Width - DefPos.X) then LineBreak;

			Canvas.TextOut(FCurPos.X, FCurPos.Y, S);
			Inc(FCurPos.X, TextWidth);
		end;
	end;
end;

function TForm_Message.ExecuteControlChar(P: PChar): Integer;

	procedure IncChar(var P: PChar);
	begin
		Inc(P, LenTable[P^ in LeadBytes]);
	end;

	function GetUntil(var P: PChar; c: Char): String;
	var
		pStart: PChar;
	begin
		pStart := P;
		while P^ <> c do IncChar(P);
    SetString(Result, pStart, p - pStart);
	end;

	function RRGGBBtoColor(const S: String): TColor;
	begin
		Result := RGB(StrToInt('$' + S[1] + S[2]),
									StrToInt('$' + S[3] + S[4]),
									StrToInt('$' + S[5] + S[6]));
	end;
 	
var
	pStart: PChar;
begin
	if P^ <> '[' then
	begin
		Result := 0;
		Exit;
	end;

	pStart := P;
	IncChar(P);
	case P^ of
		'w':
		begin
			IncChar(P);
			TimerTalk.Interval := StrToInt(GetUntil(P, ']'));
		end;
		'c':
		begin
			IncChar(P);
			Canvas.Font.Color := RRGGBBtoColor(GetUntil(P, ']'));
		end;
	end;
  Result := P - pStart + 1;
end;

procedure TForm_Message.FormPaint(Sender: TObject);
var
	Rgn: HRGN;
	OldColor: TColor;
	i: Integer;
begin
	Rgn := CreateRectRgn(0, 0, 0, 0);
	try
		OldColor := Canvas.Brush.Color;
		Canvas.Brush.Color := clInfoText;
		GetWindowRgn(Handle, Rgn);
		FrameRgn(Canvas.Handle, Rgn, Canvas.Brush.Handle, 1, 1);
		Canvas.Brush.Color := OldColor;

    FCurPos := DefPos;
		i := 1;
		while i <= Length(FText) do
			if FText[i] <> '[' then
			begin
				DrawChar(Copy(FText, i, LenTable[FText[i] in LeadBytes]));
				Inc(i, LenTable[FText[i] in LeadBytes]);
			end
			else
				Inc(i, ExecuteControlChar(@FText[i]));

	finally
		DeleteObject(Rgn);
	end;
end;

procedure TForm_Message.FormMouseDown(Sender: TObject;
	Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
	TimerTalk.Enabled := False;
	TimerTalk.Interval := 1;
	TimerTalk.Enabled := True;
	FSkip := True;
end;

procedure TForm_Message.FormMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  FSkip := False;
end;

procedure TForm_Message.FormClick(Sender: TObject);
begin
	if not TimerTalk.Enabled then Hide;
end;

end.

