unit ImoutoCLS;

interface

uses Windows, Forms, ActiveX, Dialogs, ImoutoSDK, uDispatch;

type
	TImouto = class(TDispatchImpl, IImouto)
	private
		function GetLeft: Integer;
		procedure SetLeft(Value: Integer);
		function GetTop: Integer;
		procedure SetTop(Value: Integer);

		function GetWidth: Integer;
		function GetHeight: Integer;
	protected
		function InvokeGetLeft(Params: POleVariantArray; Argc: Integer; var VarResult: OleVariant): HRESULT;
		function InvokeSetLeft(Params: POleVariantArray; Argc: Integer; var VarResult: OleVariant): HRESULT;
		function InvokeGetTop(Params: POleVariantArray; Argc: Integer; var VarResult: OleVariant): HRESULT;
		function InvokeSetTop(Params: POleVariantArray; Argc: Integer; var VarResult: OleVariant): HRESULT;
		function InvokeGetWidth(Params: POleVariantArray; Argc: Integer; var VarResult: OleVariant): HRESULT;
		function InvokeGetHeight(Params: POleVariantArray; Argc: Integer; var VarResult: OleVariant): HRESULT;
	public
		constructor Create;

		property Left: Integer read GetLeft write SetLeft;
		property Top:  Integer read GetTop  write SetTop;
		property Width: Integer read GetWidth;
		property Height: Integer read GetHeight;
	end;

	TImoutoBalloon = class(TDispatchImpl, IImoutoBalloon)
	protected
		function InvokeTalk(Params: POleVariantArray; Argc: Integer; var VarResult: OleVariant): HRESULT;
		function InvokeClear(Params: POleVariantArray; Argc: Integer; var VarResult: OleVariant): HRESULT;
	public
		constructor Create;
		function Talk(const Text: WideString): HRESULT;
		function Clear: HRESULT;
	end;

	TImoutoDesktop = class(TDispatchImpl, IImoutoDesktop)
	private
		FImouto: IImouto;
		FBalloon: IImoutoBalloon;
		function GetImouto: IImouto;
		function GetBalloon: IImoutoBalloon;
	protected
		function InvokeImouto(Params: POleVariantArray; Argc: Integer; var VarResult: OleVariant): HRESULT;
		function InvokeBalloon(Params: POleVariantArray; Argc: Integer; var VarResult: OleVariant): HRESULT;
	public
		constructor Create;
		property Imouto: IImouto read GetImouto;
		property Balloon: IImoutoBalloon read GetBalloon;
	end;

	TScreenDispatch = class(TDispatchImpl)
	protected
		function InvokeGetWidth(Params: POleVariantArray; Argc: Integer; var VarResult: OleVariant): HRESULT;
		function InvokeGetHeight(Params: POleVariantArray; Argc: Integer; var VarResult: OleVariant): HRESULT;
	public
		constructor Create;
	end;

implementation

uses Form_BalloonWindow, FormMain;

constructor TImouto.Create;
begin
	inherited;
	RegisterProp('Left',   InvokeGetLeft,   InvokeSetLeft);
	RegisterProp('Top',    InvokeGetTop,    InvokeSetTop);
	RegisterProp('Width',  InvokeGetWidth,  nil);
	RegisterProp('Height', InvokeGetHeight, nil);
end;

function TImouto.GetLeft: Integer;
begin
	Result := Form_Main.Left;
end;

procedure TImouto.SetLeft(Value: Integer);
begin
	Form_Main.Left := Value;
end;

function TImouto.GetTop: Integer;
begin
	Result := Form_Main.Top;
end;

procedure TImouto.SetTop(Value: Integer);
begin
	Form_Main.Top := Value;
end;

function TImouto.GetWidth: Integer;
begin
	Result := Form_Main.Width;
end;

function TImouto.GetHeight: Integer;
begin
	Result := Form_Main.Height;
end;

function TImouto.InvokeGetLeft(Params: POleVariantArray; Argc: Integer; var VarResult: OleVariant): HRESULT;
begin
	if @VarResult <> nil then VarResult := GetLeft;
	Result := S_OK;
end;

function TImouto.InvokeSetLeft(Params: POleVariantArray; Argc: Integer; var VarResult: OleVariant): HRESULT;
begin
	if Argc = 1 then
	begin
		Result := S_OK;
		SetLeft(Params^[0]);
	end
	else
		Result := DISP_E_BADPARAMCOUNT;
end;

function TImouto.InvokeGetTop(Params: POleVariantArray; Argc: Integer; var VarResult: OleVariant): HRESULT;
begin
	if @VarResult <> nil then VarResult := GetTop;
	Result := S_OK;
end;

function TImouto.InvokeSetTop(Params: POleVariantArray; Argc: Integer; var VarResult: OleVariant): HRESULT;
begin
	if Argc = 1 then
	begin
		Result := S_OK;
		SetTop(Params^[0]);
	end
	else
		Result := DISP_E_BADPARAMCOUNT;
end;

function TImouto.InvokeGetWidth(Params: POleVariantArray; Argc: Integer; var VarResult: OleVariant): HRESULT;
begin
	if @VarResult <> nil then VarResult := GetWidth;
	Result := S_OK;
end;

function TImouto.InvokeGetHeight(Params: POleVariantArray; Argc: Integer; var VarResult: OleVariant): HRESULT;
begin
	if @VarResult <> nil then VarResult := GetHeight;
	Result := S_OK;
end;

constructor TImoutoBalloon.Create;
begin
	inherited;
	RegisterFunc('Talk', InvokeTalk);
  RegisterFunc('Clear', InvokeClear);
end;

function TImoutoBalloon.InvokeTalk(Params: POleVariantArray; Argc: Integer; var VarResult: OleVariant): HRESULT;
begin
	if Argc = 1 then
		Result := Talk(Params^[0])
	else
		Result := DISP_E_BADPARAMCOUNT;
end;

function TImoutoBalloon.Talk(const Text: WideString): HRESULT;
begin
	Form_Message.SetMessage(Text);
	Result := S_OK;
end;

function TImoutoBalloon.InvokeClear(Params: POleVariantArray; Argc: Integer; var VarResult: OleVariant): HRESULT;
begin
	if Argc = 0 then
		Result := Clear
	else
		Result := DISP_E_BADPARAMCOUNT;
end;

function TImoutoBalloon.Clear: HRESULT;
begin
	Form_Message.ClearMessage;
  Result := S_OK;
end;

constructor TImoutoDesktop.Create;
begin
	inherited;
	FImouto  := TImouto.Create; 
	FBalloon := TImoutoBalloon.Create;
	RegisterProp('Imouto', InvokeImouto, nil);
	RegisterProp('Balloon', InvokeBalloon, nil);
end;

function TImoutoDesktop.InvokeImouto(Params: POleVariantArray; Argc: Integer; var VarResult: OleVariant): HRESULT;
begin
	if Argc = 0 then
	begin
		if @VarResult <> nil then
			VarResult := GetImouto;
		Result := S_OK;
	end
	else
		Result := DISP_E_BADPARAMCOUNT;
end;

function TImoutoDesktop.InvokeBalloon(Params: POleVariantArray; Argc: Integer; var VarResult: OleVariant): HRESULT;
begin
	if Argc = 0 then
	begin
		if @VarResult <> nil then
			VarResult := GetBalloon;
		Result := S_OK;
	end
	else
		Result := DISP_E_BADPARAMCOUNT;
end;

function TImoutoDesktop.GetImouto: IImouto;
begin
	Result := FImouto;
end;

function TImoutoDesktop.GetBalloon: IImoutoBalloon;
begin
	Result := FBalloon;
end;

constructor TScreenDispatch.Create;
begin
	inherited;
	RegisterProp('Width', InvokeGetWidth, nil);
	RegisterProp('Height', InvokeGetHeight, nil);
end;

function TScreenDispatch.InvokeGetWidth(Params: POleVariantArray; Argc: Integer; var VarResult: OleVariant): HRESULT;
begin
	if @VarResult <> nil then VarResult := Screen.WorkAreaWidth;
	Result := S_OK;
end;

function TScreenDispatch.InvokeGetHeight(Params: POleVariantArray; Argc: Integer; var VarResult: OleVariant): HRESULT;
begin
	if @VarResult <> nil then VarResult := Screen.WorkAreaHeight;
	Result := S_OK;
end;

end.

