unit uDispatch;

interface

uses Windows, Classes, ActiveX, Math;

type
	TWideCharArray = array[0..65535] of PWideChar;
	PWideCharArray = ^TWideCharArray;
	POleVariantArray = ^TOleVariantArray;
	TOleVariantArray = array[0..65535] of OleVariant;

	TInvokeMethod = function(Params: POleVariantArray; Argc: Integer; var VarResult: OleVariant): HRESULT of object;

	TInvokeItem = class(TCollectionItem)
	private
		FName: WideString;
		FGetFunc, FPutFunc: TInvokeMethod;
	public
		property Name: WideString read FName;
		function Invoke( Flags: Word; Params: POleVariantArray; Argc: Integer; VarResult: POleVariant ): HRESULT;
	end;

	TInvokeCollection = class(TOwnedCollection)
	private
		function GetItem(Index: Integer): TInvokeItem;
	public
		constructor Create(AOwner: TPersistent);
		property Items[Index: Integer]: TInvokeItem read GetItem;

		function Add: TInvokeItem;
		function Insert(Index: Integer): TInvokeItem;
	end;

	TDispatchImpl = class(TInterfacedPersistent, IDispatch)
	private
		FInvokes: TInvokeCollection;
	protected
		function GetTypeInfo(Index: Integer; LocaleID: Integer; out TypeInfo): HRESULT; stdcall;
		function GetTypeInfoCount(out Count: Integer): HRESULT; stdcall;
		function GetIDsOfNames(const IID: TGUID; Names: Pointer; NameCount: Integer; LocaleID: Integer; DispIDs: Pointer): HRESULT; stdcall;
		function Invoke(DispID: Integer; const IID: TGUID; LocaleID: Integer; Flags: Word; var Params; VarResult: Pointer; ExcepInfo: Pointer; ArgErr: Pointer): HRESULT; stdcall;

		function RegisterProp(Name: WideString; GetFunc, PutFunc: TInvokeMethod): TInvokeItem;
		function RegisterFunc(Name: WideString; Func: TInvokeMethod): TInvokeItem;
		function ItemOf(Name: WideString): TInvokeItem;
	public
		constructor Create;
		destructor Destroy; override;
	end;

type
	TDispatch = class(TInterfacedObject, IDispatch)
	private
		FImplDispatch: IDispatch;

		function GetImplAutomation: OleVariant;
	public
		constructor Create(AImplInterfae: IDispatch);
		property ImplDispatch: IDispatch read FImplDispatch implements IDispatch;
		property ImplAutomation: OleVariant read GetImplAutomation;
		function InvokeMethod(const Name: WideString; const Params: array of OleVariant): OleVariant; overload;
		function InvokeMethod(const Name: WideString): OleVariant; overload;
		function InvokeGet(const Name: WideString): OleVariant;
		procedure InvokePut(const Name: WideString; Value: OleVariant);
	end;

implementation

function TInvokeItem.Invoke( Flags: Word; Params: POleVariantArray; Argc: Integer; VarResult: POleVariant ): HRESULT;
var
	DummyResult: OleVariant;
begin
	if VarResult = nil then VarResult := @DummyResult;
		if ((Flags and DISPATCH_METHOD) = DISPATCH_METHOD) or
			 ((Flags and DISPATCH_PROPERTYGET) = DISPATCH_PROPERTYGET) then
			Result := FGetFunc( Params, Argc, VarResult^ )
		else if (Flags and DISPATCH_PROPERTYPUT) = DISPATCH_PROPERTYPUT then
			Result := FPutFunc( Params, Argc, VarResult^ )
		else
			Result := DISP_E_EXCEPTION;
end;

constructor TInvokeCollection.Create(AOwner: TPersistent);
begin
	inherited Create(AOwner, TInvokeItem);
end;

function TInvokeCollection.GetItem(Index: Integer): TInvokeItem;
begin
	Result := TInvokeItem(inherited Items[Index]);
end;

function TInvokeCollection.Add: TInvokeItem;
begin
	Result := TInvokeItem(inherited Add);
end;

function TInvokeCollection.Insert(Index: Integer): TInvokeItem;
begin
	Result := TInvokeItem(inherited Insert(Index));
end;

constructor TDispatchImpl.Create;
begin
	inherited;
	FInvokes := TInvokeCollection.Create(Self);
end;

destructor TDispatchImpl.Destroy;
begin
	FInvokes.Free;
	inherited;
end;

function TDispatchImpl.GetTypeInfo(Index: Integer; LocaleID: Integer; out TypeInfo): HRESULT;
begin
	Pointer(TypeInfo) := nil;
	Result := E_NOTIMPL;
end;

function TDispatchImpl.GetTypeInfoCount(out Count: Integer): HRESULT;
begin
	Count := 0;
	Result := S_OK;
end;

function TDispatchImpl.GetIDsOfNames(const IID: TGUID; Names: Pointer; NameCount: Integer; LocaleID: Integer; DispIDs: Pointer): HRESULT;
var
	i: Integer;
	_IDs: PDispIDList absolute DispIDs;
	_Names: PWideCharArray absolute Names;
	Item: TInvokeItem;
begin
	if NameCount > FInvokes.Count then
		Result := DISP_E_UNKNOWNNAME
	else if NameCount < 1 then
		Result := E_INVALIDARG
	else
		Result := S_OK;

	for i := 0 to NameCount - 1 do _IDs[i] := DISPID_UNKNOWN;

	for i := 0 to Min(NameCount, FInvokes.Count) - 1 do
	begin
		Item := ItemOf(_Names[i]);
		if Item <> nil then
			_IDs[i] := Item.ID
		else
      Result := DISP_E_UNKNOWNNAME;
	end;
end;

function TDispatchImpl.Invoke(DispID: Integer; const IID: TGUID; LocaleID: Integer; Flags: Word; var Params; VarResult: Pointer; ExcepInfo: Pointer; ArgErr: Pointer): HRESULT;
var
	_Params: TDispParams absolute Params;
	Item: TInvokeItem;
begin
	Item := FInvokes.Items[DispID];
	if Item <> nil then
		Result := Item.Invoke(Flags,
													POleVariantArray(_Params.rgvarg),
													_Params.cArgs,
													VarResult)
	else
		Result := DISP_E_BADCALLEE;
end;

function TDispatchImpl.RegisterProp(Name: WideString; GetFunc, PutFunc: TInvokeMethod): TInvokeItem;
begin
	Result := FInvokes.Add;
	Result.FName       := Name;
	Result.FGetFunc    := GetFunc;
	Result.FPutFunc    := PutFunc;
end;

function TDispatchImpl.RegisterFunc(Name: WideString; Func: TInvokeMethod): TInvokeItem;
begin
	Result := FInvokes.Add;
	Result.FName       := Name;
	Result.FGetFunc    := Func;
	Result.FPutFunc    := nil;
end;

function TDispatchImpl.ItemOf(Name: WideString): TInvokeItem;
var
	i: Integer;
begin
	for i := 0 to FInvokes.Count - 1 do
		if FInvokes.Items[i].Name = Name then
		begin
			Result := FInvokes.Items[i];
			Exit;
		end;
	Result := nil;
end;

constructor TDispatch.Create(AImplInterfae: IDispatch);
begin
	inherited Create;
	FImplDispatch := AImplInterfae;
end;

function TDispatch.GetImplAutomation: OleVariant;
begin
	Result := FImplDispatch;
end;

function TDispatch.InvokeMethod(const Name: WideString; const Params: array of OleVariant): OleVariant;
var
	pName: PWideChar absolute Name;
	DispID: TDispID;
	DispParams: TDispParams;
begin
	if ImplDispatch.GetIDsOfNames(GUID_NULL, @pName, 1, LOCALE_USER_DEFAULT, @DispID) = S_OK then
	begin
		DispParams.cArgs             := Length(Params);
		DispParams.rgvarg            := @Params;
		DispParams.cNamedArgs        := 0;
		DispParams.rgdispidNamedArgs := nil;
		ImplDispatch.Invoke(DispID, GUID_NULL, LOCALE_USER_DEFAULT, DISPATCH_METHOD, DispParams, @Result, nil, nil);
	end;
end;

function TDispatch.InvokeMethod(const Name: WideString): OleVariant;
begin
	Result := InvokeMethod(Name, []);
end;

function TDispatch.InvokeGet(const Name: WideString): OleVariant;
var
	pName: PWideChar absolute Name;
	DispID: TDispID;
	DispParams: TDispParams;
begin
	if ImplDispatch.GetIDsOfNames(GUID_NULL, @pName, 1, LOCALE_USER_DEFAULT, @DispID) = S_OK then
	begin
		DispParams.cArgs             := 0;
		DispParams.rgvarg            := nil;
		DispParams.cNamedArgs        := 0;
		DispParams.rgdispidNamedArgs := nil;
		ImplDispatch.Invoke(DispID, GUID_NULL, LOCALE_USER_DEFAULT, DISPATCH_PROPERTYGET, DispParams, @Result, nil, nil);
	end;
end;

procedure TDispatch.InvokePut(const Name: WideString; Value: OleVariant);
var
	pName: PWideChar absolute Name;
	DispID: TDispID;
	DispParams: TDispParams;
begin
	if ImplDispatch.GetIDsOfNames(GUID_NULL, pName, 1, LOCALE_USER_DEFAULT, @DispID) = S_OK then
	begin
		DispParams.cArgs             := 1;
		DispParams.rgvarg            := @Value;
		DispParams.cNamedArgs        := 0;
		DispParams.rgdispidNamedArgs := nil;
		ImplDispatch.Invoke(DispID, GUID_NULL, LOCALE_USER_DEFAULT, DISPATCH_PROPERTYGET, DispParams, nil, nil, nil);
	end;
end;

end.

